/* Nilo - A Network boot loader.

   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifndef __DUMP_H__
#define __DUMP_H__

/* This is generated by running configure */
#include "niloconfig.h"

/* Define the TFTP data structures */
#include "tftp.h"

/* Define the DHCP data structures */
#include "dhcp.h"

/* Define the TFTP data structures */
#include "tftp.h"

/* Define the UDP data structures */
#include "udp.h"

/* Define the ARP data structures */
#include "arp.h"

#include "pxenv.h"
#include "pxe.h"

/* DHCP option and value (cf. RFC1533) */
static char *optnames[] = {
    "DHCP_PADOPTION",
    "DHCP_SUBNET_MASK",
    "DHCP_TIMER_OFFSET",
    "DHCP_ROUTERS_ON_SUBNET",
    "DHCP_TIME_SERVER",
    "DHCP_NAME_SERVER",
    "DHCP_DNS",
    "DHCP_LOG_SERVER",
    "DHCP_COOKIE_SERVER",
    "DHCP_LPR_SERVER",
    "DHCP_IMPRES_SSERVER",
    "DHCP_RESOURCELOCATION_SERVER",
    "DHCP_HOSTNAME",
    "DHCP_BOOTFILESIZE",
    "DHCP_MERITDUMPFILE",
    "DHCP_DOMAINNAME",
    "DHCP_SWAPSERVER",
    "DHCP_ROOTPATH",
    "DHCP_EXTENTIONS_PATH",
    "DHCP_IP_FORWARDING",
    "DHCP_NONLOCAL_SOURCE_ROUTING",
    "DHCP_POLICYFILTER",
    "DHCP_MAX_DGRAMREASM_SIZE",
    "DHCP_DEFAULT_IPTTL",
    "DHCP_PATH_MTU_AGING_TIMEOUT",
    "DHCP_PATH_MTU_PLATEAU_TABLE",
    "DHCP_IFMTU",
    "DHCP_ALLSUBNETS_LOCAL",
    "DHCP_BROADCAST_ADDR",
    "DHCP_PERFORM_MASK_DISCOVERY",
    "DHCP_MASK_SUPPLIER",
    "DHCP_PERFORM_ROUTER_DISCOVERY",
    "DHCP_ROUTER_SOLICITATION_ADDR",
    "DHCP_STATIC_ROUTE",
    "DHCP_TRAILER_ENCAPSULATION",
    "DHCP_ARPCACHE_TIMEOUT",
    "DHCP_ETHERNET_ENCAPSULATION",
    "DHCP_TCP_DEFAULT_TTL",
    "DHCP_TCP_KEEPALIVE_INTERVAL",
    "DHCP_TCP_KEEPALIVE_GARBAGE",
    "DHCP_NIS_DOMAINNAME",
    "DHCP_NIS_SERVERS",
    "DHCP_NTP_SERVERS",
    "DHCP_VENDOR_SPECIFICINFO",
    "DHCP_NETBIOS_NAMESERVER",
    "DHCP_NETBIOS_DGRAMDISTSERVER",
    "DHCP_NETBIOS_NODETYPE",
    "DHCP_NETBIOS_SCOPE",
    "DHCP_XFONT_SERVER",
    "DHCP_XDISPLAY_MANAGER",
    "DHCP_REQUESTED_IP_ADDR",
    "DHCP_IPADDR_LEASE_TIME",
    "DHCP_OPTION_OVERLOAD",
    "DHCP_MESSAGE_TYPE",
    "DHCP_SERVER_IDENTIFIER",
    "DHCP_PARAM_REQUEST",
    "DHCP_MSG",
    "DHCP_MAXMSGSIZE",
    "DHCP_T1VALUE",
    "DHCP_T2VALUE",
    "DHCP_CLASS_IDENTIFIER",
    "DHCP_CLIENT_IDENTIFIER",
    "",
    "",
    "",
    "",
    "",
    "DHCP_BOOTFILE",
    "",
    "",
    "",
    "",
    "DHCP_WWWSRV:FIXME",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "DHCP_SYSARCH",
    "DHCP_NICIF",
    "",
    "",
    "DHCP_PLATFORMID",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "DHCP_PXE1_CMD_LINE",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
    "DHCP_PXE_CMD_LINE"
};

/* DHCP message types */
static char *msgtypes[] = {
    "DHCP_PADOPTION",
    "DHCP_DISCOVER",                  /* DHCP discover */
    "DHCP_OFFER",                     /* DHCP offer packet */
    "DHCP_REQUEST",                   /* DHCP request packet */
    "DHCP_DECLINE",                   /* DHCP invalid config */
    "DHCP_ACK",                       /* DHCP acknowledge packet */
    "DHCP_NAK",                       /* DHCP negative ack packet */
    "DHCP_RELEASE",                   /* DHCP release IP addr */
    "DHCP_INFORM"                     /* DHCP request information */  
};

/* ARP table indexes types */
static char *arpentries[] = {
    "ARP_CLIENT",
    "ARP_SERVER",
    "ARP_GATEWAY",
    "ARP_DNSSERVER",
    "ARP_ROOTSERVER",
    "ARP_SWAPSERVER"
};

static char *arpops[] = {
    "ARP_OP_PAD",
    "ARP_REQUEST",
    "ARP_REPLY"
};

static char *archtypes[] = {
    "STD X86",
    "PC98 X86",
    "IA64 PC",
    "DEC ALPHA",
    "ARC X86",
    "LEAN INTEL"
};

extern void dump_pxenv (pxenv_t *pxenv);
extern void dump_pxe (pxe_t *pxe);
extern void dump_dhcp_packet(dhcp_t *pkt);
extern void dump_tftp_packet(tftp_t *pkt);

extern void dump_ip_packet (struct iphdr *ip);
extern void dump_udp_packet (struct udphdr *udp);

extern void dump_arp_table (arptable_t *arptbl);
extern void dump_arp_packet (struct arprequest *pkt);
char *_inet_ntoa(unsigned int a);

#endif /* __DUMP_H__ */





