/* Nilo - A Network boot loader.

   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* This is the include generated by configuring */
#include "niloconfig.h"
#include "trace.h"

#include "pxe_types.h"

#include "nilo.h"
#include "tftp.h"
#include "tagged.h"

static union infoblock dir;
static char *segments;
static struct segheader	sh = { 0, 0, 0, 0, 0L, 0L, 0L };
static char lastseg = 0;

extern void store(u_int32_t *loadaadr, u_int8_t *s, int32_t length);
extern struct client_data *client_info;

#if 0
static void
setup_next_segment(void) 
{
    int	vlen, flags;
    char *vinfo;
    
    NILO_TRACE_FUNCTION;

    memcpy(&sh, segments, sizeof(struct segheader));
    vlen = sh.vlen;
    flags = sh.flags;
    lastseg = (sh.flags & 0x4) != 0;
    vinfo = segments + sizeof(struct segheader);

#ifdef NILO_USE_TRACING
    NILO_TRACE_MSG ("Segment header:\n");
    NILO_TRACE_PRINTF ("\t\tLoadaddr: 0x%08x\n"
                       "\t\tImagelength: 0x%04x\n"
                       "\t\tMemlength: 0x%04x\n"
                       "\t\tFlags: 0x%08x\n",
                       sh.loadaddr, sh.imglength,
                       sh.memlength, sh.flags);
    if (vlen > 0)
        NILO_TRACE_MSG ("\t\tVendor info: %s\n", vinfo);
#endif

    segments += sizeof(struct segheader) + vlen;

    NILO_TRACE_FUNCTION_RETURN;
}

static void
decode_image_header(char *data)
{
    int	i, vlen, flags;
    char *vinfo;
    
    NILO_TRACE_FUNCTION;

    memcpy(&dir, data, 512);
    vlen = dir.i.vlen;
    vinfo = (char *)&dir + sizeof(struct imgheader);
    segments = vinfo + vlen;

#ifdef NILO_USE_TRACING
    NILO_TRACE_MSG ("Tagged image header:\r\n"
           "\t\tLocation: 0x%04hx:0x%04hx\r\n\t\tExecaddr: 0x%04hx:0x%04hx\r\n",
           dir.i.location.ds, dir.i.location.bx,
           dir.i.execaddr.cs, dir.i.execaddr.ip);
    if (vlen > 0)
        NILO_TRACE_PRINTF ("\t\tVendor info: %s\r\n", vinfo);
#endif

    NILO_TRACE_FUNCTION_RETURN;
}
#endif /* if 0 */

/* handle_block() - process a tagged image file block, typically one TFTP
 *                  packet in size. (defaults to 512 bytes)
 * 
 * parameters:
 *      blockno - The block number from the TFTP packet.
 *      dirblk - The block;
 */
void
handle_block(int blockno, union infoblock *dirblk)
{
    static struct { unsigned short bx, ds; } location;
    static struct { unsigned short ip, cs; } execaddr;
    static u_int32_t *loadaddr;
    static struct imgheader *imghead;   /* pointer to the image header */
    static struct segheader *seghead;   /* pointer to the segment headers */
    static int32_t total_bytes;
    int	n, blocksize;

    NILO_TRACE_FUNCTION;

    /* If this is the first block, extract the load address for this record. */
    if (blockno == 1) {
        imghead = (struct imgheader *)dirblk;
#if 1
        /* FIXME: Malloc up some memory. This address is actually specified
           in the image header, but for running on Unix, we want to stick it
           somewhere safe. */
        loadaddr = (struct u_int32_t *)malloc (0x100000);
//        memcpy ((u_int8_t *)imghead, dirblk, sizeof(union infoblock));
#endif
        /* Extract the location and the exec address. */
        NILO_TRACE_MSG ("Storing the Image Header into memory.\r\n");
//        memcpy ((u_int8_t *)loadaddr, (u_int8_t *)dirblk, sizeof(union infoblock));
        store (loadaddr, (u_int8_t *)dirblk, sizeof(union infoblock));
        /* store the location of the image header where we can use it later,
           cause we need the segment headers to know how to load all the other
           blocks. */
        memcpy ((char *)&execaddr, (char *)&imghead->execaddr, sizeof(int));
//        imghead = (struct imgheader *)&imghead->location;

        /* Get the first segment header. */
        seghead = (struct segheader *)(((char *)loadaddr) + (imghead->vlen*4)
                                   + (imghead->nlen*4));
#if 0
        /* FIXME: We normally set the new loadaddr to the specified address,
           but since we're running this on Linux, we want to leave it pointing
           to our allocated space. */
        loadaddr = seghead->loadaddr;
#else
        loadaddr += sizeof(union infoblock);
        return;
#endif /* if 0 */
        NILO_TRACE_FUNCTION_RETURN;
    } else {
        /* Get the next segment header. Because we've long done processing the 
           image header, we look where we stashed it to read it back. */
        if ((total_bytes + client_info->tftp_blksize) > seghead->imglength) {
            /* Store only the bytes from the first segment. */
            store (loadaddr, (u_int8_t *)dirblk, seghead->imglength-total_bytes);

            /* Get the new segment header. */
            seghead = (struct segheader *)((u_int8_t *)seghead + (seghead->vlen*4) + (seghead->nlen*4));
            /* Get the new load address for this segment. */
#if 0
            /* FIXME: Normally we use the new segment header's load address,
               but for running on Linux, we just incrment the point again. */
            loadaddr = seghead->loadaddr;
#else
            loadaddr += client_info->tftp_blksize;
#endif
            /* Store the leftover bytes from this packet to the new segment's
               load address */
            store (loadaddr, (u_int8_t *)dirblk+(seghead->imglength-total_bytes),
                   client_info->tftp_blksize - (seghead->imglength-total_bytes));
            total_bytes = client_info->tftp_blksize - (seghead->imglength-total_bytes);
        } else {
            if (client_info->filetype == TAGGED_LINUX) {
                /* Decompress each packet, which is a gzipped boot image. */
                NILO_TRACE_MSG ("FIXME: do something here, should decompress packet.\r\n");
            }
            /* Write the packet to memory. */
            store (loadaddr, (u_int8_t *)dirblk, client_info->tftp_blksize);
            total_bytes += client_info->tftp_blksize;
            /* If this was the last block we needed to finish this segment,
               then get the next segment header. */
            if (total_bytes == seghead->imglength) {
                NILO_TRACE_MSG ("Done loading a segment.\r\n");
                total_bytes = 0;
                if (!seghead->endmark) {
                    seghead = (struct segheader *)((u_int8_t *)seghead + (seghead->vlen*4) + (seghead->nlen*4));
#if 0
                    /* FIXME: Normally we use the new segment header's load address,
                       but for running on Linux, we just incrment the point again. */
                    loadaddr = seghead->loadaddr;
#else
                    loadaddr += client_info->tftp_blksize;
#endif
                } else {
                    NILO_TRACE_MSG ("Done loading all segments.\r\n");
                }
            }
        }

        /* Adjust the load address for the next packet. FIXME: This is wrong,
           because it will always get the next segment header, not the current
           one. */
        switch (seghead->flags) {
        case LOAD_ABSOLUTE:
//            NILO_TRACE_PRINTF ("\t\tLoading to an absolute 32 bit address, no change.\r\n");
            loadaddr += client_info->tftp_blksize;
            break;
        case LOAD_SUB_ADDR:
//            NILO_TRACE_PRINTF ("\t\tFIXME: Loading to the last writeable location in memory.\r\n");
            break;
        case LOAD_ADD_ADDR:
//            NILO_TRACE_PRINTF ("\t\tLoad address is 1 past the last image.\r\n");
            loadaddr += client_info->tftp_blksize;
            break;
        case LOAD_SUB_START:
//            NILO_TRACE_PRINTF ("\t\tLoad address is subtracted from the
            //            start of the last image.\r\n");
            loadaddr -= client_info->tftp_blksize;
            break;
        }
    }
    
    NILO_TRACE_FUNCTION_RETURN;
}
