/*
 * Copyright (c) 1997-1998 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Flux OSKit.  The OSKit is free software, also known
 * as "open source;" you can redistribute it and/or modify it under the terms
 * of the GNU General Public License (GPL), version 2, as published by the Free
 * Software Foundation (FSF).  To explore alternate licensing terms, contact
 * the University of Utah at csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The OSKit is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with the OSKit; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 */

/* Modified from FreeBSD 2.1 sys/i386/boot/netboot */
/**************************************************************************
NETBOOT -  BOOTP/TFTP Bootstrap Program

Author: Martin Renters
  Date: Dec/93

**************************************************************************/

#ifndef __UDP_H__
#define __UDP_H__

/* This is generated by running configure */
#include "niloconfig.h"

/* Define the basic data types */
#include <sys/types.h>

/* Define the PXE types */
#include "pxe_types.h"

/* Define the tracing macros */
#include "trace.h"

/* Define the DHCP data structures */
#include "dhcp.h"

/* Define the TFTP data structures */
#include "tftp.h"

/* Define the ARP data structures */
#include "arp.h"

struct iphdr {
    char verhdrlen;
    char service;
    unsigned short len;
    unsigned short ident;
    unsigned short frags;
    char ttl;
    char protocol;
    unsigned short chksum;
    char src[4];
    char dest[4];
};

#define IP_UDP		17		/* UDP protocol */
#define IP_BROADCAST	0xFFFFFFFF

unsigned short bootp_ipchksum(void *ipv, int len);

struct udphdr {
    unsigned short src;
    unsigned short dest;
    unsigned short len;
    unsigned short chksum;
};

struct udp_pkt {
    struct iphdr ip;
    struct udphdr udp;
    union {
        dhcp_t dhcp;
        tftp_t tftp;
        struct arprequest arp;
    } u;
};

//u_int16_t dhcp_ipchksum(struct iphdr *ipv, int len);
u_int16_t dhcp_ipchksum(void *ipv, int len);

/* srcip in network order, please */
int dhcp_udp_broadcast(char *src_hwaddr, unsigned long srcip,
                       unsigned short srcsock, unsigned short destsock,
                       int len, void *bufv);

int32_t
tftp_udp_transmit(char *ether_src, u_int32_t srcip,
        u_int32_t destip, u_int32_t netmask, u_int16_t srcsock, 
	u_int16_t destsock, int32_t len, void *bufv);

#endif /* __UDP_H__ */






